(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('lodash'), require('fs'), require('tslint-folders')) :
  typeof define === 'function' && define.amd ? define(['exports', 'lodash', 'fs', 'tslint-folders'], factory) :
  (factory((global.tslintFoldersDiagrams = {}),global.lodash,global.fs,global.tslintFolders));
}(this, (function (exports,lodash,fs,tslintFolders) { 'use strict';

  var DocFormat;
  (function (DocFormat) {
      DocFormat["Text"] = "Text";
      DocFormat["Dot"] = "Dot";
  })(DocFormat || (DocFormat = {}));

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */
  /* global Reflect, Promise */

  var extendStatics = function(d, b) {
      extendStatics = Object.setPrototypeOf ||
          ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
          function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
      return extendStatics(d, b);
  };

  function __extends(d, b) {
      extendStatics(d, b);
      function __() { this.constructor = d; }
      d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  }

  var ArrayUtils;
  (function (ArrayUtils) {
      function removeFrom(arr, item) {
          var index = arr.findIndex(function (n) { return n.id === item.id; });
          if (index >= 0) {
              arr.splice(index, 1);
          }
      }
      ArrayUtils.removeFrom = removeFrom;
  })(ArrayUtils || (ArrayUtils = {}));

  var NodeType;
  (function (NodeType) {
      NodeType[NodeType["Normal"] = 0] = "Normal";
      NodeType[NodeType["External"] = 1] = "External";
      // The 'any' (*) package or folder:
      NodeType[NodeType["Any"] = 2] = "Any";
  })(NodeType || (NodeType = {}));
  var GraphNode = /** @class */ (function () {
      function GraphNode(parent, id, name, description, nodeType) {
          if (description === void 0) { description = ""; }
          if (nodeType === void 0) { nodeType = NodeType.Normal; }
          this.parent = parent;
          this.id = id;
          this.name = name;
          this.description = description;
          this.nodeType = nodeType;
          this.incomingEdges = [];
          this.outgoingEdges = [];
      }
      GraphNode.prototype.removeIncomingEdge = function (edge) {
          ArrayUtils.removeFrom(this.incomingEdges, edge);
      };
      GraphNode.prototype.removeOutgoingEdge = function (edge) {
          ArrayUtils.removeFrom(this.outgoingEdges, edge);
      };
      return GraphNode;
  }());

  var ClusterType;
  (function (ClusterType) {
      ClusterType[ClusterType["Root"] = 0] = "Root";
      ClusterType[ClusterType["TopLevel"] = 1] = "TopLevel";
      ClusterType[ClusterType["AreaWithSubFolders"] = 2] = "AreaWithSubFolders";
      ClusterType[ClusterType["FromOptimization"] = 3] = "FromOptimization";
      // from the tslint.json file 'diagramCluster':
      ClusterType[ClusterType["DiagramCluster"] = 4] = "DiagramCluster";
  })(ClusterType || (ClusterType = {}));
  var GraphCluster = /** @class */ (function (_super) {
      __extends(GraphCluster, _super);
      function GraphCluster() {
          var _this = _super !== null && _super.apply(this, arguments) || this;
          _this.nodes = [];
          _this.clusterType = ClusterType.TopLevel;
          return _this;
      }
      GraphCluster.create = function (parent, numericId, name, description, clusterType) {
          var cluster = new GraphCluster(parent, "C" + numericId, name, description);
          cluster.clusterType = clusterType;
          return cluster;
      };
      GraphCluster.prototype.removeNode = function (node) {
          ArrayUtils.removeFrom(this.nodes, node);
      };
      return GraphCluster;
  }(GraphNode));

  /**
   * Map package or folder name to a unique, non-spaced ID that can be used in dot file.
   */
  var MapNameToId = /** @class */ (function () {
      function MapNameToId() {
          this.mapNameToId = new Map();
          // must be 1-based, since is used to get next graphviz color:
          this.nextId = 1;
      }
      MapNameToId.prototype.getId = function (name) {
          if (!this.hasId(name)) {
              this.mapNameToId.set(name, this.nextId++);
          }
          var num = this.mapNameToId.get(name);
          return "P" + num;
      };
      MapNameToId.prototype.getIdOrThrow = function (name) {
          if (!this.hasId(name)) {
              throw new Error("cannot find name in map: '" + name + "'");
          }
          return this.getId(name);
      };
      MapNameToId.prototype.getNumberOrThrow = function (name) {
          if (!this.hasId(name)) {
              throw new Error("cannot find name in map: '" + name + "'");
          }
          return this.mapNameToId.get(name);
      };
      MapNameToId.prototype.hasId = function (name) {
          return this.mapNameToId.has(name);
      };
      return MapNameToId;
  }());

  var Edge = /** @class */ (function () {
      function Edge(origin, destination) {
          this.origin = origin;
          this.destination = destination;
      }
      Edge.create = function (origin, destination) {
          var edge = new Edge(origin, destination);
          origin.outgoingEdges.push(edge);
          destination.incomingEdges.push(edge);
      };
      Object.defineProperty(Edge.prototype, "id", {
          get: function () {
              return this.origin.id + "->" + this.destination.id;
          },
          enumerable: true,
          configurable: true
      });
      Edge.prototype.remove = function () {
          this.origin.removeOutgoingEdge(this);
          this.destination.removeIncomingEdge(this);
      };
      return Edge;
  }());

  var MapIdToGraphNode = /** @class */ (function () {
      function MapIdToGraphNode() {
          this.mapIdToNode = new Map();
      }
      MapIdToGraphNode.prototype.add = function (node) {
          var id = node.id;
          if (this.mapIdToNode.has(id)) {
              throw new Error("already have an entry for id " + id);
          }
          this.mapIdToNode.set(id, node);
      };
      MapIdToGraphNode.prototype.getByIdOrThrow = function (id) {
          if (!this.mapIdToNode.has(id)) {
              throw new Error("id " + id + " not found in graph map!");
          }
          return this.mapIdToNode.get(id);
      };
      return MapIdToGraphNode;
  }());

  /**
   * Generate a graph that represents the structure described in the tslint config.
   *
   * The graph can then be optimized before rendering (mainly to reduce the number of edges).
   *
   * note: the graph could contain circular references!
   */
  var GraphGenerator = /** @class */ (function () {
      function GraphGenerator(config, filter) {
          this.config = config;
          this.filter = filter;
          // the root cluster is 0 (root is not output)
          this.containerId = 0;
          this.mapNameToId = new MapNameToId();
          this.mapIdToNode = new MapIdToGraphNode();
          this.root = GraphCluster.create(null, this.containerId++, "root", "", ClusterType.Root);
          this.mapIdToNode.add(this.root);
      }
      GraphGenerator.prototype.generateGraph = function (packageConfig) {
          var _this = this;
          var _a;
          var packageFolders = packageConfig.checkImportsBetweenPackages.packages.filter(function (pkg) {
              return _this.filter.isImportPathOkForFolder(pkg);
          });
          this.checkIfNeedsAnyPackage(packageFolders);
          var parent = this.root;
          var topLevelCluster = GraphCluster.create(this.root, this.containerId++, "Top Level Packages", "", ClusterType.TopLevel);
          if (!this.filter.hasWhitelist) {
              this.root.nodes.push(topLevelCluster);
          }
          parent = topLevelCluster;
          (_a = parent.nodes).push.apply(_a, this.generateNodesFromPackages(parent, packageFolders));
          packageFolders.forEach(function (pkg) {
              _this.processPackageSubFolders(_this.root, pkg);
          });
          this.processEdges(packageFolders.filter(function (pkg) { return _this.filter.canOutputSubfoldersOf(pkg); }));
          return this.root;
      };
      GraphGenerator.prototype.checkIfNeedsAnyPackage = function (packageFolders) {
          var _this = this;
          if (!this.config.dot.showImportAnyAsNodeNotEdges) {
              return;
          }
          if (packageFolders.some(function (pkg) { return pkg.allowedToImport.some(function (allowed) { return allowed === "*"; }); })) {
              packageFolders.push(this.createAnyPackage());
          }
          // also for sub-folders:
          packageFolders.forEach(function (pkg) {
              if (pkg.subFolders.some(function (sub) { return sub.allowedToImport.some(function (allowed) { return allowed === "*"; }); })) {
                  pkg.subFolders.push(_this.createAnySubPackage());
              }
          });
      };
      GraphGenerator.prototype.generateNodesFromPackages = function (cluster, packageFolders) {
          var _this = this;
          if (!this.config.dot.clusterFromTslintJson) {
              return packageFolders.map(function (pkg) { return _this.generateNodeFromPackage(cluster, pkg); });
          }
          var defaultCluster = "(defaultCluster)";
          var packagesByCluster = lodash.groupBy(packageFolders, function (pkg) { return pkg.diagramCluster || defaultCluster; });
          var nodes = [];
          Object.keys(packagesByCluster).forEach(function (key) {
              var _a;
              var pkgs = packagesByCluster[key];
              var nodesThisCluster = pkgs.map(function (pkg) { return _this.generateNodeFromPackage(cluster, pkg); });
              if (key === defaultCluster) {
                  nodes.push.apply(nodes, nodesThisCluster);
              }
              if (key !== defaultCluster) {
                  var subCluster_1 = GraphCluster.create(cluster, _this.containerId++, "", "", ClusterType.DiagramCluster);
                  _this.mapIdToNode.add(cluster);
                  nodes.push(subCluster_1);
                  (_a = subCluster_1.nodes).push.apply(_a, nodesThisCluster);
                  nodesThisCluster.forEach(function (n) { return (n.parent = subCluster_1); });
              }
          });
          return nodes;
      };
      GraphGenerator.prototype.generateNodeFromPackage = function (cluster, pkg) {
          var packageName = pkg.importPath;
          var node = this.generateNode(cluster, packageName, pkg.description, this.getNodeType(pkg));
          return node;
      };
      GraphGenerator.prototype.getNodeType = function (pkg) {
          if (pkg.isExternal) {
              return NodeType.External;
          }
          if (pkg.importPath === this.getAnyPackageId()) {
              return NodeType.Any;
          }
          return NodeType.Normal;
      };
      GraphGenerator.prototype.getNodeTypeForSubFolder = function (pkg) {
          if (pkg.importPath === this.getAnyPackageId()) {
              return NodeType.Any;
          }
          return NodeType.Normal;
      };
      /**
       * process the sub-folders of a package as a separate graph cluster.
       *
       * Else, the graph is too hard to read.
       */
      GraphGenerator.prototype.processPackageSubFolders = function (parent, pkg) {
          var _this = this;
          var _a;
          var isContainer = pkg.subFolders.length > 0;
          if (!isContainer) {
              return;
          }
          if (!this.filter.canOutputSubfoldersOf(pkg)) {
              return;
          }
          var cluster = GraphCluster.create(parent, this.containerId++, pkg.importPath, pkg.description, ClusterType.AreaWithSubFolders);
          this.mapIdToNode.add(cluster);
          var nodes = this.generateSubFolders(cluster, pkg.importPath, pkg.subFolders.filter(function (sub) { return _this.filter.isImportPathOkForSubFolder(sub); }));
          (_a = cluster.nodes).push.apply(_a, nodes);
          this.root.nodes.push(cluster);
      };
      GraphGenerator.prototype.generateNode = function (cluster, packageName, description, nodeType, prefix) {
          if (nodeType === void 0) { nodeType = NodeType.Normal; }
          var packageIdKey = this.getPackageIdKey(packageName, prefix);
          var packageId = this.mapNameToId.getId(packageIdKey);
          var node = new GraphNode(cluster, packageId, packageName, description, nodeType);
          this.mapIdToNode.add(node);
          return node;
      };
      GraphGenerator.prototype.getPackageIdKey = function (packageName, prefix) {
          return prefix + "_" + packageName;
      };
      GraphGenerator.prototype.processEdges = function (packageFolders) {
          var _this = this;
          packageFolders.forEach(function (pkg) {
              var thisPkgId = _this.mapNameToId.getIdOrThrow(_this.getPackageIdKey(pkg.importPath));
              var allowedPackages = pkg.allowedToImport;
              if (allowedPackages.some(function (allowed) { return allowed === "*"; })) {
                  if (_this.config.dot.showImportAnyAsNodeNotEdges) {
                      allowedPackages = [_this.getAnyPackageId()];
                  }
                  else {
                      allowedPackages = packageFolders
                          // disallow import from self
                          .filter(function (allowed) { return allowed.importPath !== pkg.importPath; })
                          .map(function (allowed) { return allowed.importPath; });
                  }
              }
              _this.processEdgesForPackageNames(thisPkgId, allowedPackages);
              // TODO review - kind of duplicate code - could have interface across PackageFolder, PackageSubFolder ?
              pkg.subFolders
                  .filter(function (sub) { return _this.filter.isImportPathOkForSubFolder(sub); })
                  .forEach(function (subFolder) {
                  var thisSubFolderId = _this.mapNameToId.getId(_this.getPackageIdKey(subFolder.importPath, pkg.importPath));
                  var allowedSubFolders = subFolder.allowedToImport;
                  if (allowedSubFolders.some(function (allowed) { return allowed === "*"; })) {
                      if (_this.config.dot.showImportAnyAsNodeNotEdges) {
                          allowedSubFolders = [_this.getAnyPackageId()];
                      }
                      else {
                          allowedSubFolders = pkg.subFolders
                              .filter(
                          // disallow import from self
                          function (otherSubFolder) {
                              return otherSubFolder.importPath !== subFolder.importPath;
                          })
                              .map(function (sub) { return sub.importPath; });
                      }
                  }
                  _this.processEdgesForPackageNames(thisSubFolderId, allowedSubFolders, pkg.importPath);
              });
          });
      };
      GraphGenerator.prototype.getAnyPackageId = function () {
          return "*";
      };
      GraphGenerator.prototype.createAnyPackage = function () {
          return {
              allowedToImport: [],
              description: "(can import any packages)",
              importPath: this.getAnyPackageId(),
              isExternal: false,
              subFolders: []
          };
      };
      GraphGenerator.prototype.createAnySubPackage = function () {
          return {
              allowedToImport: [],
              description: "(can import any folders)",
              importPath: this.getAnyPackageId()
          };
      };
      GraphGenerator.prototype.processEdgesForPackageNames = function (thisPkgId, allowedPackages, packageIdPrefix) {
          var _this = this;
          allowedPackages
              .filter(function (pkg) { return _this.filter.isImportPathOkForEdges(pkg); })
              .forEach(function (allowedPkg) {
              var allowedPkgId = _this.mapNameToId.getIdOrThrow(_this.getPackageIdKey(allowedPkg, packageIdPrefix));
              var origin = _this.mapIdToNode.getByIdOrThrow(thisPkgId);
              var destination = _this.mapIdToNode.getByIdOrThrow(allowedPkgId);
              Edge.create(origin, destination);
          });
      };
      GraphGenerator.prototype.generateSubFolders = function (cluster, packageName, subFolders) {
          var _this = this;
          if (subFolders.length === 0) {
              return [];
          }
          return subFolders.map(function (folder) {
              return _this.generateNode(cluster, folder.importPath, folder.description, _this.getNodeTypeForSubFolder(folder), packageName);
          });
      };
      return GraphGenerator;
  }());

  /**
   * Track visited IDs to avoid circular references causing an infinite loop!
   */
  var VisitedIdTracker = /** @class */ (function () {
      function VisitedIdTracker() {
          this.visitedIds = new Map();
      }
      VisitedIdTracker.prototype.isVisitedOrAdd = function (id) {
          if (this.visitedIds.has(id)) {
              return true;
          }
          this.visitedIds.set(id, true);
          return false;
      };
      return VisitedIdTracker;
  }());

  var GraphVisitor = /** @class */ (function () {
      function GraphVisitor(root) {
          this.root = root;
          this.visited = new VisitedIdTracker();
      }
      GraphVisitor.prototype.visitEdges = function (visit) {
          var allEdges = [];
          var visitedEdges = new VisitedIdTracker();
          var addNodeEdges = function (node) {
              node.incomingEdges.forEach(function (edge) {
                  if (!visitedEdges.isVisitedOrAdd(edge.id)) {
                      allEdges.push(edge);
                  }
              });
              node.outgoingEdges.forEach(function (edge) {
                  if (!visitedEdges.isVisitedOrAdd(edge.id)) {
                      allEdges.push(edge);
                  }
              });
          };
          this.visitNodesAndClusters(addNodeEdges);
          allEdges.map(visit);
      };
      GraphVisitor.prototype.visitNodesAndClusters = function (visit) {
          var _this = this;
          var visitNode = function (node) {
              if (_this.visited.isVisitedOrAdd(node.id)) {
                  return;
              }
              visit(node);
              if (node instanceof GraphCluster) {
                  node.nodes.map(visitNode);
              }
          };
          visitNode(this.root);
      };
      return GraphVisitor;
  }());

  /**
   * Optimize the given graph, to reduce the number of edges so that it is more readable.
   *
   * Algorithm:
   *
   * [done] 1a. all nodes with same set of incoming edges -> place in an 'optimization' cluster, and replace those edges with 1 edge to the cluster
   * 1b. edges with same origin, and destinations are all in same *optimization* cluster -> replace with 1 edge to that cluster
   * [done] 1c. edges with same destination, and origins are all in same *optimization* cluster -> replace with 1 edge from that cluster
   * (future) could detect nodes with *mostly* same incoming edges -> place in cluster, and replace *some* edges
   * (alt option) nodes in same cluster as 'records' (just a rendering option?)
   */
  var GraphOptimizer = /** @class */ (function () {
      function GraphOptimizer() {
          this.containerId = 1;
      }
      GraphOptimizer.prototype.optimize = function (root) {
          this.optimizeNodesWithSameIncomings(root);
          // TODO xxx optimization 1b
          this.optimizeEdgesWithOriginsInSameCluster(root);
      };
      GraphOptimizer.prototype.optimizeNodesWithSameIncomings = function (root) {
          // 1a. all nodes with same set of incoming edges -> place in cluster, and replace those edges with 1 edge to the cluster
          var _this = this;
          var _a = this.populateMaps(root), mapNodeIdToNode = _a.mapNodeIdToNode, mapNodeIdToOrigins = _a.mapNodeIdToOrigins;
          // build map from 'same origin IDs' to nodes
          var mapOriginIdSetToNodeIds = new Map();
          mapNodeIdToOrigins.forEach(function (value, nodeId) {
              var originIds = value.join(",");
              if (!mapOriginIdSetToNodeIds.get(originIds)) {
                  mapOriginIdSetToNodeIds.set(originIds, []);
              }
              mapOriginIdSetToNodeIds.get(originIds).push(nodeId);
          });
          mapOriginIdSetToNodeIds.forEach(function (value, originIds) {
              var nodeIds = mapOriginIdSetToNodeIds.get(originIds);
              if (nodeIds.length > 1) {
                  // We have a set of nodes that can be clustered
                  root.nodes
                      .filter(function (node) { return node instanceof GraphCluster; })
                      .forEach(function (node) {
                      var cluster = node;
                      _this.optimizeNodesWithSameIncomingsForCluster(cluster, nodeIds, mapNodeIdToNode);
                  });
              }
          });
      };
      GraphOptimizer.prototype.populateMaps = function (root) {
          var visitor = new GraphVisitor(root);
          var mapNodeIdToOrigins = new Map();
          var mapNodeIdToNode = new Map();
          var visit = function (node) {
              mapNodeIdToNode.set(node.id, node);
              node.incomingEdges.forEach(function (edge) {
                  if (!mapNodeIdToOrigins.get(node.id)) {
                      mapNodeIdToOrigins.set(node.id, []);
                  }
                  var list = mapNodeIdToOrigins.get(node.id);
                  list.push(edge.origin.id);
              });
          };
          visitor.visitNodesAndClusters(visit);
          return {
              mapNodeIdToNode: mapNodeIdToNode,
              mapNodeIdToOrigins: mapNodeIdToOrigins
          };
      };
      GraphOptimizer.prototype.optimizeNodesWithSameIncomingsForCluster = function (parent, nodeIds, mapNodeIdToNode) {
          var _a;
          var matchingNodeIdsInParent = nodeIds.filter(function (node) {
              return parent.nodes.find(function (n) { return n.id === node; });
          });
          var nodes = matchingNodeIdsInParent.map(function (id) {
              var node = mapNodeIdToNode.get(id);
              if (!node) {
                  throw new Error("cannot find node from map for id " + id);
              }
              return node;
          });
          // remove the nodes from the parent:
          nodes.forEach(function (node) { return parent.removeNode(node); });
          // remove the old edges, and build list of origin nodes:
          var originNodes = [];
          nodes.forEach(function (node) {
              // copy list of edges, since we are deleting from it:
              var incomingEdges = node.incomingEdges.slice();
              incomingEdges.forEach(function (edge) {
                  edge.remove();
                  originNodes.push(edge.origin);
              });
          });
          // Add the new cluster:
          var clusterId = "CO1_" + this.containerId++;
          var newCluser = new GraphCluster(parent, clusterId, "");
          newCluser.clusterType = ClusterType.FromOptimization;
          (_a = newCluser.nodes).push.apply(_a, nodes);
          nodes.forEach(function (node) { return (node.parent = newCluser); });
          parent.nodes.push(newCluser);
          // add the new edges:
          originNodes.forEach(function (origin) {
              Edge.create(origin, newCluser);
          });
      };
      GraphOptimizer.prototype.optimizeEdgesWithOriginsInSameCluster = function (root) {
          // 1c. edges with same destination, and origins are all in same *optimization* cluster -> replace with 1 edge from that cluster
          // For a node, find all the edges where the origin is in the same cluster
          var mapNodeIdToNode = this.populateMaps(root).mapNodeIdToNode;
          mapNodeIdToNode.forEach(function (node) {
              if (node.incomingEdges.length < 2 || !node.parent) {
                  return;
              }
              var mapEdgesByIncomingClusterId = new Map();
              node.incomingEdges.forEach(function (edge) {
                  if (!edge.origin.parent) {
                      return;
                  }
                  if (node.parent.id === edge.origin.parent.id) {
                      // node and edge origin are in same cluster
                      return;
                  }
                  var incomingClusterId = edge.origin.parent.id;
                  if (!mapEdgesByIncomingClusterId.has(incomingClusterId)) {
                      mapEdgesByIncomingClusterId.set(incomingClusterId, []);
                  }
                  mapEdgesByIncomingClusterId.get(incomingClusterId).push(edge);
              });
              // For cluster IDs that have multiple edges, replace those edges with 1:
              mapEdgesByIncomingClusterId.forEach(function (edges, clusterId) {
                  if (edges.length < 2) {
                      return;
                  }
                  var incomingCluster = mapNodeIdToNode.get(clusterId);
                  if (!incomingCluster) {
                      throw new Error("cannot find cluster id " + clusterId);
                  }
                  if (!(incomingCluster instanceof GraphCluster)) {
                      throw new Error("incoming cluster is not a cluster! - id " + clusterId);
                  }
                  // only create an edge from that cluster, if ALL the nodes in the cluster have outgoing to this node:
                  var allClusterNodesHaveOutgoingsToThisNode = incomingCluster.nodes.every(function (n) { return n.outgoingEdges.some(function (e) { return e.destination.id === node.id; }); });
                  if (!allClusterNodesHaveOutgoingsToThisNode) {
                      return;
                  }
                  // remove the edges
                  edges.forEach(function (edge) { return edge.remove(); });
                  // add 1 edge from the cluster, to the node
                  Edge.create(incomingCluster, node);
              });
          });
      };
      return GraphOptimizer;
  }());

  var DateHelper;
  (function (DateHelper) {
      function nowHumanReadable() {
          var options = {
              year: "numeric",
              month: "long",
              day: "numeric",
              hour: "numeric",
              minute: "numeric"
          };
          var today = new Date();
          return today.toLocaleDateString(undefined, options);
      }
      DateHelper.nowHumanReadable = nowHumanReadable;
  })(DateHelper || (DateHelper = {}));

  var PackageFilter = /** @class */ (function () {
      function PackageFilter(config) {
          this.invalidPaths = this.parseBlacklist(config.importBlacklist);
          this.whiteListPaths = this.parseWhitelist(config.importWhitelist);
          this.skipSubFolders = config.skipSubFolders;
      }
      PackageFilter.prototype.parseBlacklist = function (list) {
          return list.split(",");
      };
      PackageFilter.prototype.parseWhitelist = function (list) {
          return list.split(",").filter(function (entry) { return entry.length > 0; });
      };
      PackageFilter.prototype.passesBlacklist = function (importPath) {
          return this.invalidPaths.indexOf(importPath) < 0;
      };
      PackageFilter.prototype.passesWhitelist = function (importPath) {
          return this.whiteListPaths.length === 0 || this.whiteListPaths.indexOf(importPath) >= 0;
      };
      PackageFilter.prototype.canOutputSubfoldersOf = function (folder) {
          return this.passesWhitelist(folder.importPath);
      };
      Object.defineProperty(PackageFilter.prototype, "hasWhitelist", {
          get: function () {
              return this.whiteListPaths.length > 0;
          },
          enumerable: true,
          configurable: true
      });
      PackageFilter.prototype.isImportPathOkForFolder = function (folder) {
          return this.passesBlacklist(folder.importPath);
      };
      PackageFilter.prototype.isImportPathOkForFolderWithWhitelist = function (folder) {
          return this.isImportPathOkForFolder(folder) && this.passesWhitelist(folder.importPath);
      };
      PackageFilter.prototype.isImportPathOkForSubFolder = function (folder) {
          return !this.skipSubFolders && this.passesBlacklist(folder.importPath);
      };
      PackageFilter.prototype.isImportPathOkForEdges = function (importPath) {
          return this.passesBlacklist(importPath);
      };
      return PackageFilter;
  }());

  var DocGeneratorBase = /** @class */ (function () {
      function DocGeneratorBase(config, outputter) {
          this.config = config;
          this.outputter = outputter;
          this.filter = new PackageFilter(this.config);
      }
      return DocGeneratorBase;
  }());

  var DotStyleGenerator = /** @class */ (function () {
      function DotStyleGenerator(config, outputter) {
          this.config = config;
          this.outputter = outputter;
      }
      DotStyleGenerator.prototype.outputStyling = function () {
          this.outputGraphStyle();
          this.outputDefaultNodeStyling();
      };
      DotStyleGenerator.prototype.outputGraphNodeStyle = function (nodeType) {
          switch (nodeType) {
              case NodeType.Normal:
                  break;
              case NodeType.External:
                  this.outputStylingForExternalNode();
                  break;
              case NodeType.Any:
                  this.outputStylingForAnyNode();
                  break;
              default:
                  throw new Error("unhandled NodeType " + nodeType);
          }
      };
      DotStyleGenerator.prototype.outputStylingForExternalNode = function () {
          this.outputter.outputLine("node [style=dashed]");
      };
      DotStyleGenerator.prototype.outputStylingForAnyNode = function () {
          this.outputStylingForExternalNode();
      };
      DotStyleGenerator.prototype.outputContainerStyle = function (clusterType) {
          this.outputPlaceLabelsAtTop();
          switch (clusterType) {
              case ClusterType.AreaWithSubFolders:
                  this.outputter.outputLine("node [shape=\"" + this.config.dot.subFolderShape + "\"]");
                  break;
              case ClusterType.FromOptimization:
                  this.outputOptimizedStyle();
                  break;
              case ClusterType.DiagramCluster:
                  this.outputOptimizedStyle();
                  break;
              case ClusterType.Root:
              case ClusterType.TopLevel:
                  this.outputter.outputLine("node [shape=\"" + this.config.dot.packageShape + "\"]");
                  break;
              default:
                  throw new Error("unhandled ClusterType " + clusterType);
          }
      };
      DotStyleGenerator.prototype.outputOptimizedStyle = function () {
          this.outputter.outputLines(["color = gray", "style=dashed"]);
      };
      DotStyleGenerator.prototype.outputPlaceLabelsAtTop = function () {
          this.outputter.outputLine("labelloc = t");
          this.outputter.outputLine("");
      };
      DotStyleGenerator.prototype.outputGraphStyle = function () {
          this.outputter.outputLine("graph [");
          this.outputter.increaseIndent();
          this.outputter.outputLines(["bgcolor=\"#FFFFFF\"", "fillcolor=\"#FFFFFF\""]);
          this.outputter.decreaseIndent();
          this.outputter.outputLine("]");
      };
      DotStyleGenerator.prototype.outputDefaultNodeStyling = function () {
          // ref: working_copy.dot
          this.outputter.outputLine("node [");
          this.outputter.increaseIndent();
          // ref: colorscheme = https://graphviz.gitlab.io/_pages/doc/info/colors.html
          this.outputter.outputLines([
              "labeljust=\"l\"",
              "colorscheme=\"" + this.config.dot.colorScheme + "\"",
              "style=filled",
              "fillcolor=3",
              "shape=record"
          ]);
          this.outputter.decreaseIndent();
          this.outputter.outputLines([
              "]",
              "",
              // setting both head, tail as normally only one will apply
              "edge [arrowhead=\"vee\", arrowtail=\"vee\", style=dashed, color=\"black\"]"
          ]);
      };
      return DotStyleGenerator;
  }());

  var DotDocGenerator = /** @class */ (function (_super) {
      __extends(DotDocGenerator, _super);
      function DotDocGenerator(config, outputter) {
          var _this = _super.call(this, config, outputter) || this;
          _this.config = config;
          _this.outputter = outputter;
          _this.mapNameToId = new MapNameToId();
          _this.styler = new DotStyleGenerator(config, outputter);
          return _this;
      }
      DotDocGenerator.prototype.generateDoc = function (packageConfig) {
          var _this = this;
          var graph = this.generateGraph(packageConfig);
          this.outputSectionSeparator("Header");
          this.outputHeader();
          this.outputter.outputLine("");
          this.outputSectionSeparator("Styling");
          this.styler.outputStyling();
          this.outputter.outputLine("");
          this.outputSectionSeparator("Nodes and Clusters");
          graph.nodes.forEach(function (node) {
              if (node instanceof GraphCluster) {
                  _this.outputCluster(node);
                  return;
              }
              if (node instanceof GraphNode) {
                  _this.outputGraphNode(node);
              }
          });
          this.outputter.outputLine("");
          this.outputSectionSeparator("Edges");
          this.outputEdges(graph);
          this.outputter.outputLine("");
          this.outputFooter();
      };
      DotDocGenerator.prototype.generateGraph = function (packageConfig) {
          var generator = new GraphGenerator(this.config, this.filter);
          var graph = generator.generateGraph(packageConfig);
          if (this.config.dot.isGraphOptimizerEnabled) {
              var optimizer = new GraphOptimizer();
              optimizer.optimize(graph);
          }
          return graph;
      };
      DotDocGenerator.prototype.outputHeader = function () {
          this.outputter.outputLine("/* auto-generated by tslint-folders-diagrams docs tool at " + DateHelper.nowHumanReadable() + "*/");
          this.outputter.outputLine("digraph packages {");
          this.outputter.increaseIndent();
          this.outputGraphSettings();
      };
      DotDocGenerator.prototype.outputGraphSettings = function () {
          this.outputSectionSeparator("Graph settings");
          this.outputter.outputLine("graph [");
          this.outputter.increaseIndent();
          this.outputter.outputLines([
              "// needed to allow edge to have *cluster* as a destination:",
              "compound=true",
              // NOT using concentrate, as tends to combine edges from within a cluster, with edges from the cluster,
              // so user cannot see which edges are only from the cluster.
              "label = \"" + this.config.dot.title + "\"",
              "labelloc = t",
              "",
              "//dpi = 200",
              "ranksep=0.65",
              "nodesep=0.40",
              // top to bottom:
              "rankdir=TB",
              "",
              "style=\"filled\"",
              "",
              "len=0"
          ]);
          this.outputter.decreaseIndent();
          this.outputter.outputLine("]");
      };
      DotDocGenerator.prototype.outputFooter = function () {
          this.outputter.decreaseIndent();
          this.outputter.outputLine("}");
      };
      DotDocGenerator.prototype.outputSectionSeparator = function (sectionName) {
          this.outputter.outputLine("/* " + sectionName + " ================================= */");
      };
      DotDocGenerator.prototype.outputGraphNode = function (node, prefix) {
          this.outputScopeBegin();
          this.styler.outputGraphNodeStyle(node.nodeType);
          this.outputNode(node, prefix);
          this.outputScopeEnd();
          this.outputter.outputLine("");
      };
      DotDocGenerator.prototype.outputScopeBegin = function () {
          this.outputter.outputLine("{");
      };
      DotDocGenerator.prototype.outputScopeEnd = function () {
          this.outputter.outputLine("}");
      };
      /**
       * Output the separate graph cluster.
       */
      DotDocGenerator.prototype.outputCluster = function (cluster) {
          var isContainer = cluster.nodes.length > 0;
          if (!isContainer) {
              return;
          }
          this.outputContainerNodeStart(cluster);
          this.outputSubNodes(cluster);
          this.outputContainerNodeEnd();
          this.outputter.outputLine("");
      };
      DotDocGenerator.prototype.outputContainerNodeStart = function (cluster) {
          this.outputter.outputLine("subgraph cluster_" + cluster.id + " {");
          this.outputter.increaseIndent();
          this.styler.outputContainerStyle(cluster.clusterType);
          var formattedDescription = cluster.description.length > 0 ? " - " + cluster.description : "";
          this.outputter.outputLine("label = \"" + cluster.name + formattedDescription + "\"");
      };
      DotDocGenerator.prototype.outputContainerNodeEnd = function () {
          // styling at END so is not broken by contained node styling
          this.outputter.decreaseIndent();
          this.outputter.outputLine("}");
      };
      DotDocGenerator.prototype.outputNode = function (node, prefix) {
          var packageIdKey = this.getPrefixedPackageId(node.id, prefix);
          var fillColor = this.getColorNumber(packageIdKey);
          this.outputter.outputLine(node.id + " [label=\"" + node.name + "\n" + node.description + "\" fillcolor=" + fillColor + "]");
      };
      DotDocGenerator.prototype.getColorNumber = function (packageIdKey) {
          // TODO xxx review
          if (!this.mapNameToId.hasId(packageIdKey)) {
              this.mapNameToId.getId(packageIdKey);
          }
          var packageNumber = this.mapNameToId.getNumberOrThrow(packageIdKey);
          if (packageNumber > this.config.dot.maxColors) {
              packageNumber = packageNumber % this.config.dot.maxColors;
          }
          if (packageNumber === 0) {
              // for a graphviz color scheme, the color number must be 1 based:
              packageNumber = 1;
          }
          return packageNumber;
      };
      DotDocGenerator.prototype.getPrefixedPackageId = function (packageName, prefix) {
          return prefix + "_" + packageName;
      };
      DotDocGenerator.prototype.outputEdges = function (root) {
          var _this = this;
          var visitor = new GraphVisitor(root);
          visitor.visitEdges(function (edge) { return _this.outputEdge(edge); });
      };
      DotDocGenerator.prototype.outputEdge = function (edge) {
          var destination = edge.destination;
          var origin = edge.origin;
          // set a dummy label, to make edges further apart.
          // this is to avoid edges from within a cluster, seeming to have same origin
          var extraAttribute = "label=\" \" ";
          if (edge.destination instanceof GraphCluster) {
              // the destination needs to switch to some node in the cluster:
              var nodes = edge.destination.nodes;
              if (nodes.length === 0) {
                  console.warn("// warning: The cluster " + edge.destination.id + " is empty, but is the destination for an edge.");
                  return;
              }
              destination = nodes[0];
              extraAttribute += " lhead=cluster_" + edge.destination.id;
          }
          if (edge.origin instanceof GraphCluster) {
              var nodes = edge.origin.nodes;
              if (nodes.length === 0) {
                  console.warn("// warning: The cluster " + edge.origin.id + " is empty, but is the origin for an edge.");
                  return;
              }
              origin = nodes[0];
              extraAttribute += " ltail=cluster_" + edge.origin.id;
          }
          this.outputter.outputLine(origin.id + "-> " + destination.id + " [" + extraAttribute + "]");
      };
      DotDocGenerator.prototype.outputSubNodes = function (cluster) {
          var _this = this;
          if (cluster.nodes.length === 0) {
              this.outputter.outputLine("");
              return;
          }
          cluster.nodes.forEach(function (node) {
              if (node instanceof GraphCluster) {
                  _this.outputCluster(node);
                  return;
              }
              _this.outputGraphNode(node, cluster.name);
              _this.outputter.outputLine("");
          });
      };
      return DotDocGenerator;
  }(DocGeneratorBase));

  var SECTION_SEPARATOR = "_____";
  var TextDocGenerator = /** @class */ (function (_super) {
      __extends(TextDocGenerator, _super);
      function TextDocGenerator() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      TextDocGenerator.prototype.generateDoc = function (packageConfig) {
          var _this = this;
          this.outputter.outputLine(SECTION_SEPARATOR);
          packageConfig.checkImportsBetweenPackages.packages
              .filter(function (pkg) { return !pkg.isExternal && _this.filter.isImportPathOkForFolderWithWhitelist(pkg); })
              .forEach(function (pkg) {
              var packageName = pkg.importPath;
              _this.outputter.outputLine(packageName + " - " + pkg.description);
              _this.outputter.increaseIndent();
              _this.outputAllowedImports(pkg);
              _this.outputSubFolders(pkg.subFolders.filter(function (sub) { return _this.filter.isImportPathOkForSubFolder(sub); }));
              _this.outputter.decreaseIndent();
          });
          this.outputter.outputLine(SECTION_SEPARATOR);
      };
      TextDocGenerator.prototype.outputAllowedImports = function (pkg) {
          var allowedToImport = this.outputAllowedToImport(pkg.allowedToImport);
          this.outputter.outputLine("--> " + allowedToImport);
      };
      TextDocGenerator.prototype.outputAllowedToImport = function (allowed) {
          if (allowed.some(function (value) { return value === "*"; })) {
              return "(any)";
          }
          return allowed.length > 0 ? "" + allowed.join(", ") : "(none)";
      };
      TextDocGenerator.prototype.outputSubFolders = function (subFolders) {
          var _this = this;
          if (subFolders.length === 0) {
              this.outputter.outputLine("");
              return;
          }
          this.outputter.increaseIndent();
          this.outputter.outputLine("folders:");
          this.outputter.increaseIndent();
          subFolders.forEach(function (folder) {
              _this.outputter.outputLine(folder.importPath + " - " + folder.description);
              var allowedToImport = _this.outputAllowedToImport(folder.allowedToImport);
              _this.outputter.increaseIndent();
              _this.outputter.outputLine("--> " + allowedToImport);
              _this.outputter.decreaseIndent();
              _this.outputter.outputLine("");
          });
          this.outputter.decreaseIndent();
          this.outputter.decreaseIndent();
      };
      return TextDocGenerator;
  }(DocGeneratorBase));

  var DocGeneratorFactory;
  (function (DocGeneratorFactory) {
      function create(config, outputter) {
          switch (config.format) {
              case DocFormat.Dot:
                  return new DotDocGenerator(config, outputter);
              case DocFormat.Text:
                  return new TextDocGenerator(config, outputter);
              default:
                  throw new Error("unhandled format '" + config.format + "'");
          }
      }
      DocGeneratorFactory.create = create;
  })(DocGeneratorFactory || (DocGeneratorFactory = {}));

  var SPACES_PER_TAB = 2;
  var OutputterBase = /** @class */ (function () {
      function OutputterBase() {
          this.indent = 0;
      }
      OutputterBase.prototype.decreaseIndent = function () {
          this.indent--;
          if (this.indent < 0) {
              throw new Error("indent would be negative!");
          }
      };
      OutputterBase.prototype.increaseIndent = function () {
          this.indent++;
      };
      OutputterBase.prototype.outputLine = function (text) {
          var indentText = " ".repeat(this.indent * SPACES_PER_TAB);
          this.writeLine("" + indentText + text);
      };
      OutputterBase.prototype.outputLines = function (lines) {
          var _this = this;
          lines.forEach(function (line) { return _this.outputLine(line); });
      };
      return OutputterBase;
  }());

  var ConsoleOutputter = /** @class */ (function (_super) {
      __extends(ConsoleOutputter, _super);
      function ConsoleOutputter() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      ConsoleOutputter.prototype.writeLine = function (line) {
          console.log("" + line);
      };
      return ConsoleOutputter;
  }(OutputterBase));

  var NEW_LINE = "\n";
  var FileOutputter = /** @class */ (function (_super) {
      __extends(FileOutputter, _super);
      function FileOutputter(outpath) {
          var _this = _super.call(this) || this;
          _this.outpath = outpath;
          _this.firstWrite = true;
          return _this;
      }
      /**
       * Write to file as soon as possible (in case debugging)
       *
       * note: node 'fs.sync' is not really sync writing
       * ref: https://www.daveeddy.com/2013/03/26/synchronous-file-io-in-nodejs/
       * but it simplifies the API here
       */
      FileOutputter.prototype.writeLine = function (text) {
          var line = text + NEW_LINE;
          if (fs.existsSync(this.outpath) && !this.firstWrite) {
              fs.appendFileSync(this.outpath, line);
          }
          else {
              fs.writeFileSync(this.outpath, line, "utf8");
          }
          this.firstWrite = false;
      };
      return FileOutputter;
  }(OutputterBase));

  var OutputterFactory;
  (function (OutputterFactory) {
      function create(config) {
          if (config.outpath.length > 0) {
              return new FileOutputter(config.outpath);
          }
          return new ConsoleOutputter();
      }
      OutputterFactory.create = create;
  })(OutputterFactory || (OutputterFactory = {}));

  var ConfigDefaults;
  (function (ConfigDefaults) {
      function getDefault() {
          return {
              dot: {
                  clusterFromTslintJson: false,
                  colorScheme: "pastel19",
                  isGraphOptimizerEnabled: true,
                  maxColors: 9,
                  packageShape: "oval",
                  showImportAnyAsNodeNotEdges: false,
                  subFolderShape: "folder",
                  subTitle: "Top-level Packages",
                  title: "Project Packages"
              },
              format: DocFormat.Dot,
              importBlacklist: "",
              importWhitelist: "",
              outpath: "",
              pathToTslintJson: "",
              skipSubFolders: false
          };
      }
      ConfigDefaults.getDefault = getDefault;
  })(ConfigDefaults || (ConfigDefaults = {}));

  var EnumUtils;
  (function (EnumUtils) {
      function parseDocFormat(formatString) {
          var format = formatString;
          validateDocFormat(format);
          return format;
      }
      EnumUtils.parseDocFormat = parseDocFormat;
      function validateDocFormat(format) {
          switch (format) {
              case DocFormat.Dot:
                  return format;
              case DocFormat.Text:
                  return format;
              default:
                  throw new Error("unhandled format '" + format + "'");
          }
      }
  })(EnumUtils || (EnumUtils = {}));

  var NUM_MANDATORY_ARGS = 4;
  var HelpNeededError = /** @class */ (function (_super) {
      __extends(HelpNeededError, _super);
      function HelpNeededError() {
          return _super !== null && _super.apply(this, arguments) || this;
      }
      return HelpNeededError;
  }(Error));
  var ArgsParser;
  (function (ArgsParser) {
      function getConfigFromArgs() {
          if (process.argv.length < NUM_MANDATORY_ARGS) {
              return null;
          }
          try {
              var config = ConfigDefaults.getDefault();
              updateConfigFromArgs(config);
              updateConfigFromOptionalArgs(config);
              return config;
          }
          catch (error) {
              if (error instanceof HelpNeededError) {
                  return null;
              }
              console.error(error);
              return null;
          }
      }
      ArgsParser.getConfigFromArgs = getConfigFromArgs;
      function updateConfigFromArgs(config) {
          var formatString = process.argv[3];
          var format = EnumUtils.parseDocFormat(formatString);
          Object.assign(config, {
              pathToTslintJson: process.argv[2],
              format: format
          });
      }
      function updateConfigFromOptionalArgs(config) {
          var _loop_1 = function (i) {
              var optionArg = process.argv[i];
              var optionParts = optionArg.split("=");
              var option = optionParts[0].trim();
              var value = optionParts.length > 1 ? optionParts[1].trim() : null;
              var assertHasValue = function (message) {
                  if (!value) {
                      throw new Error(message);
                  }
                  return value;
              };
              var assertHasNumericValue = function (message) {
                  var textValue = assertHasValue(message);
                  var numberValue = Number.parseInt(textValue, 10);
                  if (!isFinite(numberValue)) {
                      throw new Error(message);
                  }
                  return numberValue;
              };
              switch (option) {
                  case "-clusterFromTslintJson":
                      config.dot.clusterFromTslintJson = true;
                      break;
                  case "-colorScheme":
                      value = assertHasValue("color scheme must have a value, like: colorScheme=piyg11");
                      config.dot.colorScheme = value;
                      break;
                  case "-disableGraphOptimizer":
                      config.dot.isGraphOptimizerEnabled = false;
                      break;
                  case "-h":
                  case "-help":
                      throw new HelpNeededError();
                  case "-importBlacklist":
                      value = assertHasValue("black list must have a value, like: importBlacklist=todo-area,contact-area");
                      config.importBlacklist = value;
                      break;
                  case "-importWhitelist":
                      value = assertHasValue("white list must have a value, like: importWhitelist=todo-area,contact-area");
                      config.importWhitelist = value;
                      break;
                  case "-maxColors":
                      var maxColors = assertHasNumericValue("max colors must have a value, like: maxColors=8");
                      config.dot.maxColors = maxColors;
                      break;
                  case "-outpath":
                      value = assertHasValue("outpath must have a value, like: outpath=/tmp/my-file.dot");
                      config.outpath = value;
                      break;
                  case "-packageShape": {
                      value = assertHasValue("package shape must have a 'graphviz shape' value, like: packageShape=component");
                      config.dot.packageShape = value;
                      break;
                  }
                  case "-showImportAnyAsNodeNotEdges":
                      config.dot.showImportAnyAsNodeNotEdges = true;
                      break;
                  case "-skipSubFolders":
                      config.skipSubFolders = true;
                      break;
                  case "-subFolderShape": {
                      value = assertHasValue("sub-folder shape must have a 'graphviz shape' value, like: subFolderShape=octagon");
                      config.dot.subFolderShape = value;
                      break;
                  }
                  case "-subTitle": {
                      value = assertHasValue("sub title must have a value, like: subTitle=MyTitle");
                      config.dot.subTitle = value;
                      break;
                  }
                  case "-title": {
                      value = assertHasValue("title must have a value, like: title=MyTitle");
                      config.dot.title = value;
                      break;
                  }
                  case "":
                      break;
                  default:
                      throw new Error("unrecognised option " + optionArg);
              }
          };
          for (var i = NUM_MANDATORY_ARGS; i < process.argv.length; i++) {
              _loop_1(i);
          }
      }
  })(ArgsParser || (ArgsParser = {}));

  var TslintConfigLoader;
  (function (TslintConfigLoader) {
      function loadTslintConfig(config) {
          return new Promise(function (resolve, reject) {
              fs.readFile(config.pathToTslintJson, "utf8", function (err, data) {
                  if (err) {
                      reject(err);
                  }
                  else {
                      var json = JSON.parse(data);
                      var packageConfigJson = json.rules["tsf-folders-imports-between-packages"][1];
                      var packageConfig = tslintFolders.ConfigFactory.createForBetweenPackages([
                          packageConfigJson
                      ]);
                      resolve(packageConfig);
                  }
              });
          });
      }
      TslintConfigLoader.loadTslintConfig = loadTslintConfig;
  })(TslintConfigLoader || (TslintConfigLoader = {}));

  var UsageText;
  (function (UsageText) {
      var NEW_LINE = "\n";
      function getFormats() {
          var texts = [];
          for (var item in DocFormat) {
              if (isNaN(Number(item))) {
                  texts.push(item);
              }
          }
          return texts.join(", ");
      }
      function showUsage() {
          console.error("USAGE: index.ts <path to tslint.json> <format> [options]" + NEW_LINE +
              ("  where format is one of: " + getFormats() + NEW_LINE) +
              ("  where options can be:" + NEW_LINE) +
              ("    -help OR -h" + NEW_LINE) +
              ("    -importBlacklist=<list of packages to exclude>" + NEW_LINE) +
              ("    -importWhitelist=<list of top-level packages to include>" + NEW_LINE) +
              ("    -outpath=<path to output file>" + NEW_LINE) +
              ("    -skipSubFolders" + NEW_LINE) +
              ("  options specific to the Dot format:" + NEW_LINE) +
              ("    -clusterFromTslintJson" + NEW_LINE) +
              ("    -colorScheme=<graphviz color scheme name>" + NEW_LINE) +
              ("    -disableGraphOptimizer" + NEW_LINE) +
              ("    -maxColors=<number of colors used by colorScheme>" + NEW_LINE) +
              ("    -packageShape=<graphviz shape> (default is oval)" + NEW_LINE) +
              ("      example shapes: box|oval|octagon|component|cyclinder|box3d|folder" + NEW_LINE) +
              ("    -showImportAnyAsNodeNotEdges" + NEW_LINE) +
              ("    -subFolderShape=<graphviz shape> (default is folder)" + NEW_LINE) +
              ("    -subTitle=<sub title>" + NEW_LINE) +
              ("    -title=<title>" + NEW_LINE));
      }
      UsageText.showUsage = showUsage;
  })(UsageText || (UsageText = {}));

  function main() {
      var config = ArgsParser.getConfigFromArgs();
      if (!config) {
          UsageText.showUsage();
          return;
      }
      var outputter = OutputterFactory.create(config);
      var generator = DocGeneratorFactory.create(config, outputter);
      TslintConfigLoader.loadTslintConfig(config).then(function (packageConfig) {
          generator.generateDoc(packageConfig);
      });
  }
  main();

  var index = /*#__PURE__*/Object.freeze({

  });

  exports.tools = index;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=tslint-folders-diagrams.umd.js.map
